/***************************************************************************
                          knutupsdata.h  -  description
                             -------------------
    begin                : Tue Aug 21 2001
    copyright            : (C) 2001 by Daniel Prynych
    email                : Daniel.Prynych@alo.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNUTUPSDATA_H
#define KNUTUPSDATA_H

#include "knutconst.h"
#include "knutanalog.h"

#include <qstring.h>
#include <qvaluelist.h>

class QString;

/**
  *@author Daniel Prynych
  */

struct upsRecord {  //include information abou UPS
  QString name; // name
  QString upsName; // name of UPSky <> mojeups
  QString upsAddress; // address of UPS <> muj.pc.cz
  long delay;  // time between read date from UPS
  unsigned int port; // port for connecting server
  QString userName;
  QString password;
  bool savePassword; //save password into configuration 
  int upsVarCode[knc::NumberOfVars];
  };


 /**
  * Trida obsahuje data o UPS-kach,
  * nazev, adresu port, protokol, promenne atd.
  * Class includes data about UPSes like name. address, port, protocol, variables ...
  *
  * @author Daniel Prynych
  * @short Information about UPS
  * @version 0.3
  **/
class KNutUpsData {

public:
 /**
  * Konstruktor
  *
  * @since  0.2
  **/
  KNutUpsData();

 /**
  * Destruktor
  *
  * @since  0.2
  **/
  ~KNutUpsData();

 /**
  * Adds record
  * Prida zaznam.
  *
  * @param upsMember are data of record in structure of UpsInfo.
  * @param upsMember Jsou data zaznamu ve strukture UpsInfo.
  * @since  0.3
  **/
  void add (const upsRecord upsMember);

 /**
  * Prepise zaznam.
  *
  * @param index Udava poradove cislo zaznamu..
  * @param upsMember Jsou data zaznamu ve strukture UpsInfo.
  * @since  0.3
  **/
  void put (const int index, const upsRecord upsMember );

 /**
  * Vrati zaznam.
  *
  * @param index Udava poradove cislo zaznamu.
  * @param upsMember Jsou data zaznamu ve strukture UpsInfo.
  *
  * @since  0.3
  **/
  void get (const int index, upsRecord& upsMember );

 /**
  * Vrati jmeno zaznamu.
  *
  * @param index Udava poradove cislo zaznamu.
  * @since  0.2
  **/
  QString getName (const int index);

 /**
  * Deletes record;
  * Zrusi zaznam.
  *
  * @param index Describes order number od record.
  * @param index Udava poradove cislo zaznamu.
  *
  * @since  0.2
  **/
  void deleteName (const int index);

 /**
  * Returns count of records
  * Vrati pocet zaznamu.
  *
  * @since  0.2
  **/
  int getCount ( void );

 /**
  * Returns pointer on record, when record is not existed returns 0
  * Vrati ukazatel na zaznam, pokud zaznam neexistuje vrati 0.
  *
  * @param name Is name if list.
  * @param name Je jmeno zaznamu.
  * @since  0.2
  **/
  upsRecord* findName (const QString name);



private:
  int m_countMembers;

 QValueList<upsRecord> m_listRecords;
};


#endif
